-- Model: CA in geodatabeses - Raining on the mountain 
--
-- Author: Pedro Andrade & Tiago Carneiro
--
-- Lesson: Unsig geographical databases in TerraME
--

--------------------------------------------------------------
-- GLOBAL 
--------------------------------------------------------------

tmePath = os.getenv("TME_PATH_1_3_0")
databasePath = tmePath.."/database/"
--------------------------------------------------------------
-- MODEL 
--------------------------------------------------------------

-------------
--# SPACE #
-------------

cb = CellularSpace{
	database = databasePath.."cabecaDeBoi_4_2_0.mdb",
	theme = "cells90x90",
	select = {"height_ as height", "soilWater"}
}


forEachCell(cb, function(cell)
	cell.soilWater = 0
end)

----------------
--# BEHAVIOR #
----------------
function rain()
	forEachCell(cb,function(cell)
		if cell.height > 200 then
			cell.soilWater = cell.soilWater + 500
		end
	end)
end

-----------
--# TIME #
-----------

t = Timer{
	
	--Event{period = 0.5, action = function(e) -- (2) try to replace the line below by this line when the action returning value is true
	Event{action = function(ev)
		rain()
		--return false -- (1) try to comment this line
	end},
	Event{priority = 5, action = function(ev)
		print( "Simulation time:", ev:getTime() )
		cb:notify()
	end}
}

--------------------------------------------------------------
-- SIMULATION
--------------------------------------------------------------

soilWaterLeg = Legend{
	grouping = "equalsteps",
	slices = 20,
	colorBar = {
		{color = "white", value = 0},
		{color = "blue", value = 5000}
	}
}

obs1 = Observer{
	subject = cb,
	type = "map",
	attributes = {"soilWater"},
	legends= {soilWaterLeg}
}

cb:notify()

t:execute(5)

