-- Model: CA in geodatabeses - Rain drainage model 
--
-- Author: Pedro Andrade & Tiago Carneiro
--
-- Lesson: Unsig geographical databases in TerraME
--

--------------------------------------------------------------
-- GLOBAL 
--------------------------------------------------------------

tmePath = os.getenv("TME_PATH_1_3_0")
databasePath = tmePath.."/database/"
--------------------------------------------------------------
-- MODEL 
--------------------------------------------------------------

-------------
--# SPACE #
-------------

cb = CellularSpace{
	database = databasePath.."cabecaDeBoi_4_2_0.mdb",
	theme = "cells90x90",
	select = {"height_ as height", "soilWater"}
}

cb:createNeighborhood{
	strategy = "3x3",
	filter = function(cell, neighbor)
		return cell ~= neighbor and cell.height >= neighbor.height
	end
}

forEachCell(cb, function(cell)
	cell.soilWater = 0
end)

----------------
--# BEHAVIOR #
----------------
function rain()
	forEachCell(cb,function(cell)
		if cell.height > 200 then
			cell.soilWater = cell.soilWater + 500
		end
	end)
end

function drainage()
		cb:synchronize()
		
		forEachCell(cb, function(cell)
			cell.soilWater = 0
		end)
		
		forEachCell(cb, function(cell)
			if cell:getNeighborhood():size() > 0 then
				flow = cell.past.soilWater / cell:getNeighborhood():size()
				forEachNeighbor(cell, function(cell, neigh)
					neigh.soilWater = neigh.soilWater + flow
				end)
			else
				cell.soilWater = cell.soilWater + cell.past.soilWater
			end
		end)
end

-----------
--# TIME #
-----------

t = Timer{
	
	Event{action = function(e)
		rain()
		return false 
	end},

	Event{action = function(e)
		drainage()
	end},

	Event{priority = 5, action = function(ev)
		print( "Simulation time:", ev:getTime() )
		cb:notify()
	end}
}

--------------------------------------------------------------
-- SIMULATION
--------------------------------------------------------------

heightLeg = Legend{
	grouping = "equalsteps",
	slices = 20,
	colorBar = {
		{color = "black", value = 0},
		{color = "white", value = 300}
	}
}

soilWaterLeg = Legend{
	grouping = "equalsteps",
	slices = 20,
	colorBar = {
		{color = "white", value = 0},
		{color = "blue", value = 10000}
	}
}

obs1 = Observer{
	subject = cb,
	type = "map",
	attributes = {"soilWater", "height"},
	legends= {soilWaterLeg, heightLeg}
}

cb:notify()

t:execute(100)